@@ -244,6 +244,9 @@ int intel_ucode_compare(const void * const uc1, const void * const uc2)
 	if (unlikely((hdr1->hdrver != 1) || (hdr2->hdrver != 1)))
 		return -EINVAL;
 
+	if (unlikely(uc1 == uc2))
+		return 2;
+
 	ts1 = intel_ucode_v1_get_totalsize(hdr1);
 	ts2 = intel_ucode_v1_get_totalsize(hdr2);
 	if (ts1 != ts2)
@@ -285,6 +288,7 @@ int intel_ucode_compare(const void * const uc1, const void * const uc2)
  */
 const char * intel_ucode_errstr(const intel_ucode_status_t status)
 {
+	/* warning: this is an __attribute__((const)) function! */
 	switch (status) {
 	case INTEL_UCODE_INVALID_DATA:
 		return "invalid microcode data";
@@ -311,7 +315,7 @@ const char * intel_ucode_errstr(const intel_ucode_status_t status)
 	}
 }
 
-static inline int is_zero_checksum(const uint8_t *data, uint32_t dwords)
+static int is_zero_checksum(const uint8_t *data, uint32_t dwords)
 {
 	uint32_t s = 0;
 
