@@ -1405,7 +1405,7 @@ static void log_microcode_action(const char * const action,
 	print_msg_u("%s: %s microcode " UCODE_ID_FMT_UU
 		    " (sig 0x%08x, pf_mask 0x%02x, rev 0x%04x)",
 		    devname, action, uce->gid, uce->id,
-		    uce->cpuid, uce->pfm, uce->uc_rev);
+		    uce->cpuid, uce->pfm, (uint32_t) uce->uc_rev);
 }
 
 /**
@@ -1550,9 +1550,9 @@ static int xx_write_cpio_hdrentry(int fd, const char * const name,
 	  "%08X%08X%08X%08X%08X%08X%08zX%08X%08X%08X%08X%08zX%08X%s",
 	  ino,                                                     /* inode */
 	  size ? LINUX_CPIO_FILE_MODE : LINUX_CPIO_DIR_MODE,        /* mode */
-	  0, 0,                                                 /* uid, gid */
-	  size ? 1 : 2, t, size,                      /* nlink, mtime, size */
-	  3, 1, 0, 0, nsize, 0,                   /* devj, devm, nsize, CRC */
+	  0U, 0U,                                               /* uid, gid */
+	  size ? 1U : 2U, t, size,                    /* nlink, mtime, size */
+	  3U, 1U, 0U, 0U, nsize, 0U,              /* devj, devm, nsize, CRC */
 	  name);                                               /* name, pad */
 
 	return write_data(fd, buf, bufsize);
@@ -2030,8 +2030,34 @@ static int xx_xtsdeduplist_add(struct intel_uclist_entry * const e,
 	return 0;
 }
 
-static int xx_xtsdeduplist_check_and_add(struct intel_uclist_entry * const e,
-					 struct microcode_id_entry **list)
+/**
+ * xtsdeduplist_check_and_add() - track mcu objects for dedup
+ *
+ * @e:    entry to add signature from to the list
+ * @list: pointer to the head of the list used to track duplicates
+ *
+ * Duplicate tracking is done by storing and comparing struct
+ * intel_uclist_entry->uc pointers.  This will work _only_ to dedup
+ * several instances of the same microcode due to extended signature
+ * processing.
+ *
+ * It cannot detect multiple copies of the same data stored in
+ * multiple objects.
+ *
+ * To use:
+ * Init the list head to NULL.  Call xtsdeduplist_check_and_add() for
+ * every struct intel_uclist_entry element you want to track.
+ *
+ * One should use free_xtsdeduplist() to free the tracking list after
+ * use.
+ *
+ * Returns:
+ *   ENOMEM: cannot allocate memory to add entry to list
+ *   EEXIST: entry is a duplicate
+ *   0:      entry is not a duplicate
+ */
+static int xtsdeduplist_check_and_add(struct intel_uclist_entry * const e,
+                                      struct microcode_id_entry **list)
 {
 	const void *id = e->uc;
 
@@ -2043,7 +2069,14 @@ static int xx_xtsdeduplist_check_and_add(struct intel_uclist_entry * const e,
 	return xx_xtsdeduplist_add(e, list);
 }
 
-static void xx_free_xtsdeduplist(struct microcode_id_entry *list)
+/**
+ * free_xtsdeduplist() - frees mcu dedup tracking list
+ * @list:   head of the list to be freed
+ *
+ * Frees every element of the list.  @list will be invalid
+ * after this function returns.
+ */
+static void free_xtsdeduplist(struct microcode_id_entry *list)
 {
 	struct microcode_id_entry *e;
 
@@ -2066,7 +2099,7 @@ static int uclist_annotate_extsig_dup(struct intel_uclist_entry * const uclist)
 	e = uclist;
 	while (e) {
 		if (unlikely(e->flags & INTEL_UCLE_HASXST)) {
-			rc = xx_xtsdeduplist_check_and_add(e, &dlist);
+			rc = xtsdeduplist_check_and_add(e, &dlist);
 			if (rc == EEXIST) {
 				e->flags |= INTEL_UCLE_NOWR;
 			} else if (!rc) {
@@ -2078,7 +2111,7 @@ static int uclist_annotate_extsig_dup(struct intel_uclist_entry * const uclist)
 		e = e->next;
 	}
 
-	xx_free_xtsdeduplist(dlist);
+	free_xtsdeduplist(dlist);
 	return (rc != EEXIST) ? rc : 0;
 }
 
@@ -2151,7 +2184,8 @@ static int check_downgrade_shadowing(const struct intel_uclist_entry * uclist)
 					  "pf_mask 0x%02x (entries: " UCODE_ID_FMT_UU
 					  " rev 0x%x, and " UCODE_ID_FMT_UU " rev 0x%x)",
 					  sig, (pfm & e->pfm),
-					  gid, uclist->id, rev, e->gid, e->id, e->uc_rev);
+					  gid, uclist->id, (uint32_t) rev,
+					  e->gid, e->id, (uint32_t) e->uc_rev);
 				res = EEXIST;
 			}
 			e = e->next;
@@ -2202,18 +2236,20 @@ static int xx_process_ucode_signature_cb(void * const userdata,
 
 	if (list_all_microcodes) {
 		if (!sig_count)
-			printf("  " UCODE_ID_FMT_UU ": sig 0x%08x, pf_mask 0x%02x, "
+			fprintf(stdout,
+				"  " UCODE_ID_FMT_UU ": sig 0x%08x, pf_mask 0x%02x, "
 				"%04x-%02x-%02x, rev 0x%04x, size %u\n",
 				ctx->current_bundle->id, ctx->current_uc,
 				cpuid, pf_mask,
 				m.date_year, m.date_month, m.date_day,
-				m.revision, uc_size);
+				(uint32_t) m.revision, uc_size);
 		else
-			printf("           sig 0x%08x, pf_mask 0x%02x, "
+			fprintf(stdout,
+				"           sig 0x%08x, pf_mask 0x%02x, "
 				"%04x-%02x-%02x, rev 0x%04x\n",
 				cpuid, pf_mask,
 				m.date_year, m.date_month, m.date_day,
-				m.revision);
+				(uint32_t) m.revision);
 	}
 
 	add_status = uclist_add_signature(ctx->current_uc,
@@ -2300,7 +2336,7 @@ static int do_process_microcodes(void)
 
 	while (mcb) {
 		if ((list_all_microcodes || list_sel_microcodes) && verbosity > 0)
-			printf("microcode bundle %u: %s\n", mcb->id,
+			fprintf(stdout, "microcode bundle %u: %s\n", mcb->id,
 				mcb->filename ? mcb->filename : "");
 
 		microcode_iterator_data.current_bundle = mcb;
@@ -2334,11 +2370,12 @@ static int do_process_microcodes(void)
 	if (list_sel_microcodes || datefilter_loose || verbosity >= 2) {
 		struct intel_uclist_entry *uce = microcodes;
 		struct intel_uclist_entry *ucl = all_microcodes;
+		struct microcode_id_entry *dl = NULL;
 		unsigned long int uccount = 0;
 		unsigned long int sigcount = 0;
 
 		if (list_sel_microcodes)
-			printf("selected microcodes:\n");
+			fprintf(stdout, "selected microcodes:\n");
 
 		while (uce) {
 			/*
@@ -2349,8 +2386,10 @@ static int do_process_microcodes(void)
 			    xx_datefilter_loose_inplaceinsert(&uce, &ucl)))
 					return 1;
 
-			if (likely(!(uce->flags & INTEL_UCLE_EXTSIG)))
-			    uccount++;
+			/* note: we ignore ENOMEM results for performance */
+			if (likely(!(uce->flags & INTEL_UCLE_HASXST) ||
+				   !xtsdeduplist_check_and_add(uce, &dl)))
+				uccount++;
 
 			sigcount++;
 
@@ -2359,15 +2398,18 @@ static int do_process_microcodes(void)
 
 				if (unlikely(intel_ucode_getmetadata(uce->uc, &m) != INTEL_UCODE_NOERROR))
 					exit(EXIT_SWFAILURE); /* memory corruption */
-				printf("  " UCODE_ID_FMT_UU ": sig 0x%08x, pf_mask 0x%02x, "
+				fprintf(stdout,
+					"  " UCODE_ID_FMT_UU ": sig 0x%08x, pf_mask 0x%02x, "
 					"%04x-%02x-%02x, rev 0x%04x, size %u\n",
 					uce->gid, uce->id, uce->cpuid, uce->pfm,
 					m.date_year, m.date_month, m.date_day,
-					m.revision, uce->uc_size);
+					(uint32_t) m.revision, uce->uc_size);
 			}
 
 			uce = uce->next;
 		}
+
+		free_xtsdeduplist(dl);
 		print_msg(2, "selected %lu microcode(s), %lu signature(s)",
 			  uccount, sigcount);
 	}
@@ -2449,7 +2491,7 @@ static int do_write_named(const char * const dirname,
 	while (p && !rc) {
 		if (!(p->flags & INTEL_UCLE_DUPSIG)) {
 			snprintf(fn, sizeof(fn), "s%08X_m%08X_r%08X.fw",
-				     p->cpuid, p->pfm, p->uc_rev);
+				     p->cpuid, p->pfm, (uint32_t) p->uc_rev);
 
 			memcpy(&e, p, sizeof(e));
 			e.next = NULL;
